{This program models the spatial population dynamics of a marine organism    }
{with a two-phase life cycle.  The benthic adult phase is distributed along a}
{straight coast that can be broken up into 15 units of arbitary length, each }
{with different properties.  The equations describing the population dynamics}
{of the adult phase are                                                      }
{                                                                            }
{         dB(y,t)/dt = cF(y,t)L(0,y,t) - (adultdeathrate)B(y,t)                             }
{                                                                            }
{         A = F(y,t) + aB(y,t) .                                             }
{                                                                            }
{In the water column the movement of the larvae is governed by the two       }
{dimensional transport equation                                              }
{                                                                            }
{         dL + udL + vdL  =  Kd2L + Kd2L - (larvaldeathrate)L                                }
{         dx    dx +  dy      dx2    dy2                                     }
{                                                                            }
{with boundary condition at x = 0                                            }
{                                                                            }
{         KdL = cFL(0,y,t) - m(y,t)B(y,t)                                    }
{          dx                                                                }
{                                                                            }
{The stability conditions are                                                }
{                                                                            }
{         diffusion*delt/sqr(delx) < 1/4                                     }
{                                                                            }
{            u < diffusion*4/delx             (and similarly for v)          }
{                                                                            }
{In terms of the standard parameter set where diffusion = 360, these         }
{conditions force  u (and v) < 360.  To understand the influence of stronger }
{currents the grid-spacing must be reduced and hence delt must be reduced    }
{These conditions restrict the size of the time step AND the size of any     }
{currents relative to the diffusion coefficient                              }


Program barnacle(input,output,dataout,larvaeplot,adultplot);

const bigfixj = 24;       {Defines number of grids in the x direction}
      bigfixk = 64;      {Defines number of grids in the y direction}
      blocks  = 15;

type flowarray = array [0..bigfixj,0..bigfixk] of real;
     {Type for larval population and parameters}
     xarray = array [0..bigfixj] of real;
     yarray = array [0..bigfixk] of real;
     {Type for adult population and parameters}
     yblock = array [1..blocks] of integer;
     {Type for defining coastal structure}
     densarray = array [1..blocks] of real;    {Type for coastal values}

var i,j,k,runs :integer;     {Runs is the no. of time intervals program runs}
    bigj,bigk  :integer;     {The actual number of grid lines used}
    printouts,printdist,printnum,flag1,analytic                :integer;
     {Printouts is the number of times the data is printed out}
    delt,delx,dely,diffusion,u,v,larvaldeath,barnarea,tdeath   :real;
    freespace,flowin,new1                                      :real;
    rx,s1x,s2x,xflow,yflow,ry,s1y,s2y,old,new,lainit           :flowarray;
    s3,s4,s5,s32x,s32y,s42x,s42y,s32xy,s42xy                   :flowarray;
     {xflow and yflow define the flow fields}
    dataout,larvaeplot,adultplot                               :text;
    best                                                       :xarray;
    adult,initial,settle,prod,adeath,spacey,adinit             :yarray;
    ymarks                                                     :yblock;
    density,settlecoeff,production,adultdeath,spaceorA         :densarray;
    xflowflag,yflowflag,initialize                             :boolean;


Procedure setupconstants(diffusion,larvaldeath,delt,delx,dely:real;
                         var s1x,s2x,s1y,s2y,s3,s4,s32x,s42x,
                         s32y,s42y,s32xy,s42xy,s5:flowarray);

{This procedure sets up the constant arrays that depend on the flow field}
{and are used repeatedly in the finite difference scheme}

var j,k:integer;
    sx,sy,ldt,sxsyp,sxsyldt,dtoverdx,dtoverdy:real;

begin
  writeln('You are entering the procedure "setupconstants"');
  sx:=diffusion*delt/sqr(delx);
  sy:=diffusion*delt/sqr(dely);
  writeln('Check 2: delt = ',delt:10);
  ldt:=larvaldeath*delt;
  sxsyp:=sx+sy;
  sxsyldt:=sx+sy+ldt/2;
  dtoverdx:=delt/(delx*2);
  dtoverdy:=delt/(dely*2);
  for j:= 0 to bigj do
    for k:= 0 to bigk do
      begin
        rx[j,k]:=xflow[j,k]*dtoverdx;
        s1x[j,k]:=sx+rx[j,k];
        s2x[j,k]:=sx-rx[j,k];
        ry[j,k]:=yflow[j,k]*dtoverdy;
        s1y[j,k]:=sy+ry[j,k];
        s2y[j,k]:=sy-ry[j,k];
        s3[j,k]:=1/(1+sxsyldt+rx[j,k]+ry[j,k]);
        s4[j,k]:=1-sxsyldt+rx[j,k]+ry[j,k];
        s32x[j,k]:=1/(1+sxsyldt-rx[j,k]+ry[j,k]);
        s42x[j,k]:=1-sxsyldt-rx[j,k]+ry[j,k];
        s32y[j,k]:=1/(1+sxsyldt+rx[j,k]-ry[j,k]);
        s42y[j,k]:=1-sxsyldt+rx[j,k]-ry[j,k];
        s32xy[j,k]:=1/(1+sxsyldt-rx[j,k]-ry[j,k]);
        s42xy[j,k]:=1-sxsyldt-rx[j,k]-ry[j,k];
        s5[j,k]:=1-sxsyp-sxsyp-ldt;
      end;
end;

Procedure paramgo(diffusion,u,v,larvaldeath,barnarea,delt,delx,dely:real;
                  runs,bigj,bigk,printouts:integer; ymarks:yblock;
                  density,settlecoeff,production,adultdeath,spaceorA:densarray;
                  settle,prod,adeath,spacey:yarray;
                  xflow,yflow:flowarray;var analytic:integer);

{The procedure "paramgo" writes out all the parameters of the model, }
{including a bit of the flow field, to the file "dataout".           }

var st1,st2,i,j,k,flow:integer;
    fhat,lhat,bhat,st3,st4,st5:real;

begin
  writeln('You are now entering the procedure "paramgo"');
  writeln(dataout,'The diffusion coefficient is ',(diffusion/10000):10,
                  ' square kilometres per hour');
  writeln(dataout,'The convection rate in the x direction is ',(u/1000):10,
                  ' kiloms. per hour');
  writeln(dataout,'The convection rate in the y direction is ',(v/1000):10,
                  ' kiloms. per hour');
  writeln(dataout,'The larval death rate is  ',(larvaldeath/100):10,' %/hour');
  writeln(dataout,'Size of barnacles is ',barnarea:10,' per unit space');
  writeln(dataout,'The time grid is       ',(delt*100):10,' hours');
  writeln(dataout,'The space grid in x is ',(delx*100):10,' metres. ');
  writeln(dataout,'The space grid in y is ',(dely*100):10,' metres');
  write(dataout,'The number of runs in time is ',runs:2);
  writeln(dataout,' hence total run time is ',(runs*delt*100):10,' hours');
  write(dataout,'No. of x grids is ',bigj:2,' hence distance');
  writeln(dataout,' covered from shore is ',(bigj*delx*100):10,' metres');
  write(dataout,'No. of y grids is ',bigk:2,'  hence distance');
  writeln(dataout,' covered along coast is ',(bigk*dely*100):10,' metres');
  writeln('The number of blocks is',blocks:4);
  if initialize then writeln(dataout,'You have used the standard initial data');
  for i:= 1 to blocks do
    begin
      writeln(dataout,'Block ',i:1,' is ',(ymarks[i]*delx):3,' metres long');
      writeln(dataout,' with initial density',density[i]:9);
      writeln(dataout,' settle =',settlecoeff[i]:9,' prod =',production[i]:9);
      writeln(dataout,' adeath =',adultdeath[i]:9,' space/100m =',spaceorA[i]:9);
    end;
  writeln(dataout);
  writeln(dataout,'                             Distance from shore in metres');
  st1:=trunc((bigj/8)+0.001);
  st2:=trunc(st1*delx*100);
  writeln(dataout,'              adultpop',0:6,'    ',st2:8,(2*st2):9,
                  (3*st2):9,(4*st2):9,(7*st2):9);
  writeln(dataout);
  readln(analytic);
  writeln('You will look at the adult population at point ',analytic:2,' along the coast');
  writeln('Check: your current values for larvaldeath,diffusion and barnarea are:');
  writeln('       ',larvaldeath:10,diffusion:10,barnarea:10);
  st3:=sqrt(larvaldeath*diffusion);
  st4:=settle[analytic]*spacey[analytic]*(prod[analytic]-adeath[analytic]);
  bhat:=spacey[analytic]*(1-(adeath[analytic]*st3/st4))/barnarea;
  fhat:=spacey[analytic] - (barnarea*bhat);
  lhat:=bhat*prod[analytic]/(settle[analytic]*fhat+st3);
  st5:=delx*sqrt(larvaldeath/diffusion);
  write(dataout,'             ',bhat:9,lhat:9);
  write(dataout,lhat*exp(-st1*st5):9,lhat*exp(-2*st1*st5):9);
  write(dataout,lhat*exp(-3*st1*st5):9,lhat*exp(-4*st1*st5):9);
  writeln(dataout,lhat*exp(-7*st1*st5):9);
  writeln('The 1-D solution at equilibrium is:');
  write('             ',bhat:9,lhat:9);
  write(lhat*exp(-st1*st5):9,lhat*exp(-2*st1*st5):9);
  writeln(lhat*exp(-3*st1*st5):9,lhat*exp(-4*st1*st5):9);
  readln(flow);

     {This final section writes out some values of the flow field to dataout}

  writeln(dataout,'Flow field, for u out from coast at point ',flow:5);
  writeln(dataout,'            ',xflow[0,flow]:9,xflow[1,flow]:9,xflow[2,flow]:9);
  writeln(dataout,xflow[3,flow]:9,xflow[4,flow]:9,xflow[5,flow]:9);
  writeln(dataout,'                      ',xflow[0,flow]:9,xflow[st1,flow]:9);
  writeln(dataout,xflow[2*st1,flow]:9,xflow[3*st1,flow]:9,xflow[4*st1,flow]:9);
  writeln(dataout,'Flow field, for v out from coast at point ',flow:5);
  writeln(dataout,'            ',yflow[0,flow]:9,yflow[1,flow]:9,yflow[2,flow]:9);
  writeln(dataout,yflow[3,flow]:9,yflow[4,flow]:9,yflow[5,flow]:9);
  writeln(dataout,'                      ',yflow[0,flow]:9,yflow[st1,flow]:9);
  writeln(dataout,yflow[2*st1,flow]:9,yflow[3*st1,flow]:9,yflow[4*st1,flow]:9);
  writeln(dataout);
end;


Procedure datain(var diffusion,u,v,larvaldeath,barnarea,delt,delx,dely:real;
                 var runs,bigj,bigk,printouts:integer;
                 var initial,settle,prod,adeath,spacey,adinit:yarray;
                 var ymarks:yblock;
                 var density,settlecoeff,production,adultdeath,spaceorA:densarray;
                 var xflow,yflow,lainit:flowarray; var initialize:boolean);

{Procedure "datain" reads all the data in from standard input}

var i,j,k,ymarktotal,runstodate,count:integer;
    flag,flag1,flag2,biglarvx,biglarvy,start,stop:integer;
    Reyx,Reyy,store:real;

begin
  writeln('You are now beginning the data entry phase of the program');
  writeln('First the fixed non-dimensional parameters are entered');
  readln(diffusion,larvaldeath,barnarea,delt,delx,dely);
  readln(runs,bigj,bigk,printouts);
  writeln('Now we are going to enter the characteristics of the coastline');
  writeln('  for the adult population');
  writeln('The coastline is being divided into ',blocks:1,' blocks to which');
  writeln('  the following parameter values may be assigned:');
  writeln('  larval density, settlement coeff., prod. rate, adult death rate');
  ymarktotal:=0;
  for i:= 1 to blocks do    {Reads in constants for each coastal block in turn}
    begin
      writeln('You are reading in the parameters for the ',i:1,'th block');
      readln(ymarks[i],density[i],settlecoeff[i],production[i],
             adultdeath[i],spaceorA[i]);
      writeln('Intialize from ',ymarktotal:2,' to ',(ymarktotal+ymarks[i]-1):2);
        for k:= ymarktotal to (ymarktotal+ymarks[i]-1) do
          begin
            settle[k]:=settlecoeff[i];
            prod[k]:=production[i];
            adeath[k]:=adultdeath[i];
            initial[k]:=density[i];
            spacey[k]:=spaceorA[i];
          end;
      ymarktotal:=ymarktotal+ymarks[i];
      writeln('Ymarktotal assigned so far is ',ymarktotal:4);
    end;
    if (ymarktotal > (bigk+1)) then writeln('WARNING: too many coastal sites defined');
    if (ymarktotal < (bigk+1)) then writeln('WARNING: not enough sites defined');

    writeln('You are now entering the flow field properties');
    readln(u,v);
    writeln('You are about to enter the u flow field');
    readln(flag);
    case flag of
      1:begin
          readln(Reyx,start,stop);
          writeln(dataout,'U, Reyx, start, stop: ',U:8,Reyx:8,start:4,stop:4);
          for j:= 0 to bigj do
            for k:= 0 to bigk do
              xflow[j,k]:=0;
          for j:=0 to bigj do
            for k:= start to stop do
              begin
                store:=-j*delx/Reyx;
                xflow[j,k]:=u*(1-exp(store));
              end;
         end;
       2:begin
           writeln(dataout,'U is constant at ',u:8,' metres per hour');
           for j:= 0 to bigj do
             for k:= 0 to bigk do
               xflow[j,k]:=u;
           readln;
         end;
       end;
    writeln('You are about to enter flow field props for y');
    readln(flag);
    case flag of
      1:begin
         readln(Reyy,start,stop);
         writeln('V, Reyy, start, stop: ',V:8,Reyy:8,start:4,stop:4);
         for j:= 0 to bigj do
           for k:= 0 to bigk do
             yflow[j,k]:=0;
         for j:=0 to bigj do 
           for k:= start to stop do
             begin
               store:=-j*delx/Reyy;
               yflow[j,k]:=v*(1-exp(store));
             end;
        end;
      2:begin
          writeln(dataout,'V is constant at ',v:8,' metres per hour');
          for j:= 0 to bigj do
            for k:= 0 to bigk do
              yflow[j,k]:=v;
              readln;
        end;
      end;

  writeln(dataout,' Flow field at 0,0 is ',xflow[0,0]:10,yflow[0,0]:10);
  writeln(dataout,' Flow field at 2,2 is ',xflow[2,2]:10,yflow[2,2]:10);
  writeln(dataout,' Flow field at 4,4 is ',xflow[4,4]:10,yflow[4,4]:10);
  writeln(dataout,' Flow field at 6,6 is ',xflow[6,6]:10,yflow[6,6]:10);
  writeln(dataout,' Flow field at 8,8 is ',xflow[8,8]:10,yflow[8,8]:10);
  writeln(dataout,' Flow field at 0,0 is ',xflow[16,16]:10,yflow[16,16]:10);

     {The section below reads in the initial conditions}

  readln(flag); initialize:= false;
  writeln('You are about to enter the initial conditions');
  if flag = 1 then
                begin
                  initialize:=true;
                  readln(biglarvx,biglarvy);
                  writeln('Your input file is ',biglarvx:2,' by ',biglarvy:2);
                  readln;
                  if biglarvy = bigk then writeln('y dimension in adults OK');
                  for k:= 0 to bigk do
                    readln(adinit[k]);
                  readln(biglarvx,biglarvy);
                  if biglarvx = bigj then writeln('x dimension in larvae OK');
                  if biglarvy = bigk then writeln('y dimension in larvae OK');
                  readln(runstodate);
                  for k:= 0 to bigk do
                    begin
                      count:=0;
                      for i:= 1 to 8 do
                        begin
                          for j:= 1 to 6 do
                            begin
                              read(lainit[count,k]);
                              count:=count+1;
                            end;
                          readln;
                        end;
                      readln(lainit[count,i]);
                      writeln('You have read in initial larvae, row ',k:2);
                    end;
                end;
end;                        {End procedure "datain"}


Procedure datago(var printnum:integer;i:integer;delt:real);

{This procedure sends the adult population density distribution, and the larval}
{population desnity distribution to the files: "adultplot" and "larvaeplot"    }
{respectively.  The procedure may be invoked as frequently as the user wishes  }
{these data sets to be stored: currently the program is set up so that the     }
{adult distribution is printed out "printnum" times, while the larval density  }
{is seen only when printnum is 1,2,4,8,16 or 32}

var st1,st2,j,k,l,count:integer;

begin
  printnum:=printnum+1;
  writeln('You are now entering the procedure datago after ',i:2,' runs');
  writeln(dataout,'The approx. soln. after ',trunc(i*delt*100):2,' hours is');
  st1:=trunc((bigj/8)+0.001);
  writeln(larvaeplot,bigj,bigk);
  writeln(larvaeplot,i);
  writeln(adultplot,0,bigk);
  writeln(adultplot,i);
  for k:= 0 to bigk do
    begin
      writeln(adultplot,adult[k]:12);
      if (k mod 4) = 0 then
        begin
          write(dataout,(k*0.1*dely):9,'km ',adult[k]:9,new[0,k]:9,new[st1,k]:9);
          write(dataout,new[(2*st1),k]:9,new[(3*st1),k]:9,new[(4*st1),k]:9);
          writeln(dataout,new[(7*st1),k]:9); 
        end;
      if (printnum = 1) or (printnum = 2) or (printnum = 4)
      or (printnum = 8) or (printnum = 16) or (printnum = 32) then
        begin
          count:=0;
          for l:= 1 to 8 do
            begin
              for j:= 1 to 6 do
                begin
                  write(larvaeplot,new[count,k]:12);
                  count:=count+1;
                end;
              writeln(larvaeplot);
            end;
          writeln(larvaeplot,new[count,k]:12);
        end;
    end;
end;                        {End procedure "datain"}


begin            {Beginnning of MAIN PROGRAM}

  rewrite(dataout);rewrite(larvaeplot);rewrite(adultplot);

  datain(diffusion,u,v,larvaldeath,barnarea,delt,delx,dely,runs,bigj,bigk,
         printouts,initial,settle,prod,adeath,spacey,adinit,ymarks,density,settlecoeff,
         production,adultdeath,spaceorA,xflow,yflow,lainit,initialize);

     {The data has now been read in from standard input}

  writeln('Check 1: delt before paramgo = ',delt:10);

  paramgo(diffusion,u,v,larvaldeath,barnarea,delt,delx,dely,
          runs,bigj,bigk,printouts,ymarks,
          density,settlecoeff,production,adultdeath,spaceorA,
          settle,prod,adeath,spacey,xflow,yflow,analytic);

  writeln('Check a: delt after paramgo =  ',delt:10);

  printdist:=trunc(runs/printouts);
  printnum:=0;
  flag1:=1;

          {Setting up constants and constant arrays used in program}

  setupconstants(diffusion,larvaldeath,delt,delx,dely,
                 s1x,s2x,s1y,s2y,s3,s4,s32x,s42x,s32y,s42y,s32xy,s42xy,s5);

  writeln('Check b: delt after setupconstants = ',delt:10);

                   {Initializing the larval and adult arrays}

  if initialize then           {If initialize is true then the initial    }
    for k:= 0 to bigk do       {conditions are read in from the input file}
      begin
        adult[k]:=adinit[k];
        for j:= 0 to bigj do
          begin 
            old[j,k]:=lainit[j,k];
            new[j,k]:=lainit[j,k];
          end;
      end
                 else          {Otherwise the larval density is set to zero}
    for k:= 0 to bigk do       {everywhere and the adult density is defined}
      begin                    {according to the block structure}
        adult[k]:=initial[k];
        for j:= 0 to bigj do
          begin
            old[j,k]:=0;
            new[j,k]:=0;
          end;
      end;

  for i:= 1 to (runs+1) do    {Beginning of the main finite difference loop}
    begin                           

      if (i <> 1) then
        if ((i-1) mod printdist) = 0 then
          datago(printnum,i,delt);                     {Data stored in files}

      for k:= 0 to bigk do          {Fixing the absorbing boundary condition}
                        begin                            {far from the coast}
                          new[bigj,k]:=0;
                          old[bigj,k]:=0;
                        end; 

      for j:= 0 to bigj do         {Fixing the absorbing boundary conditions}
        begin                      {at the North and South edges of the grid}
          new[j,0]:=0;     old[j,0]:=0;
          new[j,bigk]:=0;  old[j,bigk]:=0;
        end;

      for k:= 1 to (bigk-1) do        {Fixing the coastal boundary condition}
        begin                                    
          freespace:=(spacey[k]-(barnarea*adult[k]));
          new1:=old[1,k] - (2*delx*settle[k]*old[0,k]*freespace/diffusion) +
                (2*delx*prod[k]*adult[k]/diffusion);
          flowin:=(s1x[0,k]*new1) + (s1y[0,k]*old[0,k-1]) + 
                  (s2y[0,k]*old[0,k+1]) + (s2x[0,k]*old[1,k]);
          new[0,k]:=flowin + (s5[0,k]*old[0,k]);
        end;

{The boundary conditions have been fixed and we are about to enter the}
{finite difference approximation to determine the larval concentration}
{at the next time step.  The eight different variations of the semi-implicit}
{finite difference scheme are carried out sequentially in an order that}
{eliminates numerical diffusion}

        {The sweep from the coast out to the ocean south to north}

    case flag1 of 
      1:begin
          for j:= 1 to (bigj-1) do
            for k:= 1 to (bigk-1) do
              new[j,k]:=(s1x[j,k]*new[j-1,k] + s1y[j,k]*new[j,k-1]
                         + s4[j,k]*old[j,k] + s2x[j,k]*old[j+1,k] 
                         + s2y[j,k]*old[j,k+1])*s3[j,k];
          flag1:=8;
        end;

        {The sweep from the ocean into the coast south to north}

      2:begin
          for j:= (bigj-1) downto 1 do
            for k:= 1 to (bigk-1) do
              new[j,k]:=(s1x[j,k]*old[j-1,k] + s1y[j,k]*new[j,k-1]
                         + s42x[j,k]*old[j,k] + s2x[j,k]*new[j+1,k] 
                         + s2y[j,k]*old[j,k+1])*s32x[j,k];
          flag1:=7;
        end;

        {The sweep from coast out to the ocean north to south}

      3:begin
          for k:= (bigk-1) downto 1 do
            for j:= 1 to (bigj-1) do
              new[j,k]:=(s1x[j,k]*new[j-1,k] + s1y[j,k]*old[j,k-1]
                         + s42y[j,k]*old[j,k] + s2x[j,k]*old[j+1,k] 
                         + s2y[j,k]*new[j,k+1])*s32y[j,k];
          flag1:=6;
        end;

        {The sweep from the ocean in to the coast north to south}

      4:begin
          for k:= (bigk-1) downto 1 do
            for j:= (bigj-1) downto 1 do
              new[j,k]:=(s1x[j,k]*old[j-1,k] + s1y[j,k]*old[j,k-1]
                         + s42xy[j,k]*old[j,k] + s2x[j,k]*new[j+1,k] 
                         + s2y[j,k]*new[j,k+1])*s32xy[j,k];
          flag1:=5;
        end;

        {The sweep from north to south, coast to ocean}

      5:begin
          for k:= 1 to (bigk-1) do
            for j:= 1 to (bigj-1) do
              new[j,k]:=(s1x[j,k]*new[j-1,k] + s1y[j,k]*new[j,k-1]
                         + s4[j,k]*old[j,k] + s2x[j,k]*old[j+1,k] 
                         + s2y[j,k]*old[j,k+1])*s3[j,k];
          flag1:=1;
        end;

        {The sweep from the ocean into the coast south to north}

      6:begin
          for k:= 1 to (bigk-1) do
            for j:= (bigj-1) downto 1 do
              new[j,k]:=(s1x[j,k]*old[j-1,k] + s1y[j,k]*new[j,k-1]
                         + s42x[j,k]*old[j,k] + s2x[j,k]*new[j+1,k] 
                         + s2y[j,k]*old[j,k+1])*s32x[j,k];
          flag1:=4;
        end;

        {The sweep from the coast out to the ocean, north to south}

      7:begin
          for j:= 1 to (bigj-1) do
            for k:= (bigk-1) downto 1 do
              new[j,k]:=(s1x[j,k]*new[j-1,k] + s1y[j,k]*old[j,k-1]
                         + s42y[j,k]*old[j,k] + s2x[j,k]*old[j+1,k] 
                         + s2y[j,k]*new[j,k+1])*s32y[j,k];
          flag1:=3;
        end;

        {The sweep from the ocean in to the coast north to south}

      8:begin
          for j:= (bigj-1) downto 1 do
            for k:= (bigk-1) downto 1 do
              new[j,k]:=(s1x[j,k]*old[j-1,k] + s1y[j,k]*old[j,k-1]
                         + s42xy[j,k]*old[j,k] + s2x[j,k]*new[j+1,k] 
                         + s2y[j,k]*new[j,k+1])*s32xy[j,k];
          flag1:=2;
        end;
    end;

{The finite difference scheme for updating the adult barnacle population}

  for k:= 0 to bigk do
    begin
      freespace:=delt*settle[k]*(spacey[k]-(barnarea*adult[k]));
      tdeath:=adeath[k]*delt;
      adult[k]:=adult[k]*(1-tdeath) + (freespace*old[0,k]);
    end;

    writeln(' You have just completed run number ',i:2);
    writeln(' At point 36 along the coast Adult: ',adult[36]:10);
    writeln(' At point 50 along the coast Adult: ',adult[50]:10);
    writeln(' At point 64 along the coast Adult: ',adult[64]:10);
    writeln(' At point 78 along the coast Adult: ',adult[78]:10);
    writeln(' At point 92 along the coast Adult: ',adult[92]:10);

    for j:= 0 to bigj do            {Updates the larval distribution}
      for k:= 0 to bigk do
        old[j,k]:=new[j,k]; 

  end;          {End of "for i:= 1 to runs}

end.             {End of the MAIN program}
